/* $Id: vga_fb.inc,v 1.8 1998/11/23 20:37:12 marcus Exp $
******************************************************************************

   IBM VGA FB operators

   Copyright (C) 1996-1997 Jason McMullan	[jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

   This code was derived from the following sources of information:
   [1] Xfree86 VGA driver
   [2] Zen of Graphics Programming (Michael Abrash)

******************************************************************************
*/

#ifndef _VGA_FB_INC_
#define _VGA_FB_INC_

static void vgafb_set_origin(struct kgi_display *dpy, kgi_uint x, kgi_uint y)
{
	IS_CRITICAL;
	static kgi_uint last_xdot = -1;
	static kgi_uint last_ydot = -1;
	static kgi_uint old_offset = -1;
	register kgi_uint xpix=0,xdot=0;
	register kgi_uint ypix=0,ydot=0;
	register kgi_uint offset;
	kgi_uint fb_width=0;

	switch (dpy->mode.request.graphtype) {
		case KGIGT_TEXT16:
			xpix = x / dpy->mode.request.dpp.x;
			xdot = x % dpy->mode.request.dpp.x;
			ypix = y / dpy->mode.request.dpp.y;
			ydot = y % dpy->mode.request.dpp.y;
			fb_width = dpy->mode.request.virt.x * 2;
			break;
		case KGIGT_1BIT:
		case KGIGT_4BIT:
			xpix = x / 8;
			xdot = x % 8;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 8;
			break;
		case KGIGT_8BIT:
		/* ModeX and 320x200 planar seem to be the same ... becka */
			xpix = x / 4;
			xdot = (x % 4) * 2;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virt.x / 4;
		break;
		default:
		INTERNAL_ERROR;
	}
		
 	offset = xpix + ypix * fb_width;

	ENTER_CRITICAL;

	if (old_offset != offset) {

		outbCRT(offset >> 8, CRT_STARTADDR_H);
		outbCRT(offset, CRT_STARTADDR_L);
		old_offset = offset;
	}

	if (last_ydot != ydot) {
		outbCRT(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register kgi_uint hpan;

		if (dpy->mode.request.graphtype == KGIGT_TEXT16 &&
		    dpy->mode.request.dpp.x == 9) {
			
			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {
			
			hpan = xdot & AR13_DATA_MASK;
		}
		WAIT_HVRETRACE();
		outbATC(hpan, ATC_HPAN);
		outb( ATCI_ENABLEDISPLAY, ATCI);
		last_xdot = xdot;
	}
	
	LEAVE_CRITICAL;
}

static void vgafb_set_split(struct kgi_display *dpy, int line)
{
	register kgiu8 foo;

	outbCRT(line & 0xFF, CRT_LINECOMPARE);

	foo = (line & 0x0100)
		? inbCRT(0x07) | CR07_LINECOMPARE_B8
		: inbCRT(0x07) & ~CR07_LINECOMPARE_B8;
	outbCRT(foo, 0x07);

	foo = (line & 0x0200)
		? inbCRT(0x09) | CR09_LINECOMPARE_B9
		: inbCRT(0x09) & ~CR09_LINECOMPARE_B9;
	outbCRT(foo, 0x09);
}

#define	vgafb_set_frame	NULL	/* !!! implement! */

#endif /* _VGA_FB_INC */
