/*******************************************************************************
 * ATI Mach64 integrated ramdac driver
 *******************************************************************************
 *
   Copyright (C) 1998 Isaac Connor,
                      Mathew Wilkins 
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *
 * $Log: mach64.c,v $
 * Revision 1.9  1998/11/23 21:55:27  iconnor
 * Corrected, Added License info to all Mach64 files.
 * Added Me to maintainers list
 *
 * Revision 1.8  1998/11/19 02:43:04  marcus
 * License change
 *
 * Revision 1.7  1998/11/01 22:45:56  brian
 * Changed INTERNALERROR to INTERNAL_ERROR and removed INTERNALERROR
 * and INVALID_PARAMS from system.h
 *
 * Revision 1.6  1998/10/25 20:03:23  brian
 * Changed cmd to *cmd in ramdac_check_mode and accel_check_mode, plus
 * spot-cleaned a few places Marcus missed during namespace cleanup.
 * (Will get chipset/clock stuff soon, working around Marcus.)
 *
 * Revision 1.5  1998/10/25 18:04:09  marcus
 * More namespace cleanups
 *
 * Revision 1.4  1998/10/25 02:05:40  marcus
 * Namespace cleanup to make KGI/LibGGI interaction possible
 *
 * Revision 1.3  1998/10/04 03:16:38  brian
 * Added private data hook to KGI_COMMON -- clean and recompile kgicon!
 *
 * Revision 1.2  1998/08/25 05:13:07  iconnor
 * Removed include ggi/maintainers.h ... from mach64 source files
 *
 * Revision 1.1  1998/08/25 04:56:36  iconnor
 * Initial import of the new Mach64 code.  Now does detection, and mode setting.
 * This driver is semi-almost working.  Very little works.
 *
 * Revision 1.4  1998/08/24 05:37:16  wilco
 * Disabled the feature connector with the Dac_Cntl register.
 *
 * Revision 1.3  1998/08/23 06:43:25  wilco
 * Added a quick hack of a DAC palette initializer.
 *
 * Revision 1.2  1998/08/23 01:55:36  iconnor
 * Added a note about my card.
 *
 * Revision 1.1.1.1  1998/08/22 03:35:58  iconnor
 * Yo
 *
 * Revision 1.1  1998/07/07 07:51:02  taylorj
 * *** empty log message ***
 *
 * Revision 1.1  1998/01/08 08:03:39  ursetto
 * Initial check-in.
 *
 *
 *******************************************************************************
 * 
 * This code was derived from the following source of information:
 * [1]  IBM VGA driver
 * 
 */

#define	DRIVER_NAME	"ATI Technologies Mach64 integrated DAC driver"
#define	DRIVER_REV	"$Revision: 1.9 $"
#define	DEBUG_LEVEL	0

#include <kgi/kgi.h>
#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#define	DAC_PRECISION	6

#include "ATI/mach64.h"
#include "ATI/mach64_io.h"



/* Mach64 GT's can handle 170MHz at 16 bit */

static struct kgi_ramdac ramdac = 
{
	"ATI Technologies Inc.",		/* Manufacturer		*/
	"Mach64 Internal RAMDAC",		/* Model		*/
	GM_8BIT					/* Supported modes	*/
	| GM_15BIT | GM_16BIT | GM_24BIT | GM_32BIT,
	FL_DAC_8BIT,		 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	NULL,				/* Private data hook	*/
	DAC_8BIT_PB,  			/* Device type		*/
	{     0,  135000000 },		/* Normal mode		*/
	{     0,         0 },		/* 2xclk mode		*/
	{
	{	  0,  	     0 },	/* KGIGT_TEXT16	*/
	{	  0,         0 },	/* KGIGT_TEXT32	*/
	{         0, 	     0 },	/* KGIGT_1BIT	*/
	{	  0, 135000000 },	/* KGIGT_4BIT	*/
	{     	  0, 135000000 },	/* KGIGT_8BIT	*/	
	{	  0,  80000000 },	/* KGIGT_15BIT	*/
	{ 	  0,  80000000 },	/* KGIGT_16BIT	*/
	{         0,  80000000 },	/* KGIGT_24BIT	*/
	{	  0,  80000000 }	/* KGIGT_32BIT	*/
	}
};

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " " DRIVER_REV);
	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,	enum kgi_tm_cmd *cmd)
{
	uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {
		case KGIGT_4BIT:
		case KGIGT_8BIT:
		case KGIGT_15BIT:
		case KGIGT_16BIT:
		case KGIGT_24BIT:
		case KGIGT_32BIT:
			break;
		default:
			DEBUG2("couldn't handle graphtype %i", 
				   mode->request.graphtype);
			return -E(RAMDAC,MODE_CHECK);
	}

	switch (*cmd) {

	case CMD_PROPOSE:
		mode->lclk.mul = mode->clk.mul = 1;
		mode->lclk.div = mode->clk.div = 1;
		return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {
			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy,struct kgi_mode *mode)
{
	kgiu8 cnt;
	kgiu32 temp;
	switch (mode->request.graphtype) {
		case KGIGT_1BIT:
		case KGIGT_TEXT16:
		case KGIGT_4BIT:
		case KGIGT_8BIT:
		case KGIGT_15BIT:
		case KGIGT_16BIT:
		case KGIGT_24BIT:
		case KGIGT_32BIT:
			break;
		
		default:
			INTERNAL_ERROR;
	}

		temp = mach64_in32(Dac_Cntl, dpy);
		mach64_out32(Dac_Cntl,(1<<8)|/*(1<<10)|*/
					(0xFFFFD7FF & temp), dpy); 
		mach64_out8(Dac_Regs, 2,  0xFF, dpy); 	

	/* initialize the CLUT.   Not  a final solution, but handy
	 for testing other parts of the driver.  */
		mach64_out8(Dac_Regs, 0,  0, dpy); 	
		for (cnt=0;cnt<0xFF; cnt++)
		{
			mach64_out8(Dac_Regs, 1,  cnt, dpy); 	
			mach64_out8(Dac_Regs, 1,  cnt, dpy); 	
			mach64_out8(Dac_Regs, 1,  cnt, dpy); 	
		}
		
/*	dpy->set_clut = dac_set_clut;
*/
}

int kgim_ramdac_command(struct kgi_graphic *graph, kgi_uint cmd, kgi_uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	kgi_clut clut;
	return EOK;
	switch (cmd) { 

		case RAMDAC_GETINFO:
			graph->io_res = &ramdac;
			return EOK;
		
		case RAMDAC_SETINFO:
			if (suser()) {
				memcpy(&ramdac, graph->io_buf, sizeof(ramdac));
				return EOK;
			} 
			else {
				return -EPERM;
			}
		case RAMDAC_SETCLUT:
			clut.size = 256;
			clut.data = (kgi_color *)(graph->io_buf);
	/*		dac_set_clut(graph->dev.dpy, &clut, 0, 256); */
			return 0;

		case RAMDAC_GETCLUT:
			clut.size = 256;
			clut.data = (kgi_color *)(graph->io_buf);
			graph->io_res = graph->io_buf;
	/*		dac_get_clut(graph->dev.dpy, &clut, 0, 256); */
			return 0;
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}
